@@ -1,7 +1,7 @@
 /*
- * oAuth string functions in POSIX-C.
+ * OAuth http functions in POSIX-C.
  *
- * Copyright 2007, 2008, 2009 Robin Gareus <robin@gareus.org>
+ * Copyright 2007, 2008, 2009, 2010 Robin Gareus <robin@gareus.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -100,11 +100,13 @@ ReadMemoryCallbackAndCall(void *ptr, size_t size, size_t nmemb, void *data) {
  *
  * @param u url to retrieve
  * @param p post parameters 
+ * @param customheader specify custom HTTP header (or NULL for none)
  * @return returned HTTP
  */
-char *oauth_curl_post (const char *u, const char *p) {
+char *oauth_curl_post (const char *u, const char *p, const char *customheader) {
   CURL *curl;
   CURLcode res;
+  struct curl_slist *slist=NULL;
 
   struct MemoryStruct chunk;
   chunk.data=NULL;
@@ -116,8 +118,17 @@ char *oauth_curl_post (const char *u, const char *p) {
   curl_easy_setopt(curl, CURLOPT_POSTFIELDS, p);
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
+  if (customheader) {
+    slist = curl_slist_append(slist, customheader);
+    curl_easy_setopt(curl, CURLOPT_HTTPHEADER, slist); 
+  }
   curl_easy_setopt(curl, CURLOPT_USERAGENT, OAUTH_USER_AGENT);
+#ifdef OAUTH_CURL_TIMEOUT  
+  curl_easy_setopt(curl, CURLOPT_TIMEOUT, OAUTH_CURL_TIMEOUT);
+  curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
+#endif
   res = curl_easy_perform(curl);
+  curl_slist_free_all(slist);
   if (res) {
     return NULL;
   }
@@ -132,11 +143,13 @@ char *oauth_curl_post (const char *u, const char *p) {
  *
  * @param u url to retrieve
  * @param q optional query parameters 
+ * @param customheader specify custom HTTP header (or NULL for none)
  * @return returned HTTP
  */
-char *oauth_curl_get (const char *u, const char *q) {
+char *oauth_curl_get (const char *u, const char *q, const char *customheader) {
   CURL *curl;
   CURLcode res;
+  struct curl_slist *slist=NULL;
   char *t1=NULL;
   struct MemoryStruct chunk;
 
@@ -153,6 +166,10 @@ char *oauth_curl_get (const char *u, const char *q) {
   curl_easy_setopt(curl, CURLOPT_URL, q?t1:u);
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
+  if (customheader) {
+    slist = curl_slist_append(slist, customheader);
+    curl_easy_setopt(curl, CURLOPT_HTTPHEADER, slist); 
+  }
 #if 0 // TODO - support request methods..
   if (0) 
     curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, "HEAD");
@@ -160,7 +177,12 @@ char *oauth_curl_get (const char *u, const char *q) {
     curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, "DELETE");
 #endif
   curl_easy_setopt(curl, CURLOPT_USERAGENT, OAUTH_USER_AGENT);
+#ifdef OAUTH_CURL_TIMEOUT  
+  curl_easy_setopt(curl, CURLOPT_TIMEOUT, OAUTH_CURL_TIMEOUT);
+  curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
+#endif
   res = curl_easy_perform(curl);
+  curl_slist_free_all(slist);
   if (q) free(t1);
   if (res) {
     return NULL;
@@ -214,7 +236,12 @@ char *oauth_curl_post_file (const char *u, const char *fn, size_t len, const cha
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
   curl_easy_setopt(curl, CURLOPT_USERAGENT, OAUTH_USER_AGENT);
+#ifdef OAUTH_CURL_TIMEOUT  
+  curl_easy_setopt(curl, CURLOPT_TIMEOUT, OAUTH_CURL_TIMEOUT);
+  curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
+#endif
   res = curl_easy_perform(curl);
+  curl_slist_free_all(slist);
   if (res) {
     // error
     return NULL;
@@ -280,7 +307,12 @@ char *oauth_curl_send_data_with_callback (const char *u, const char *data, size_
   else 
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
   curl_easy_setopt(curl, CURLOPT_USERAGENT, OAUTH_USER_AGENT);
+#ifdef OAUTH_CURL_TIMEOUT  
+  curl_easy_setopt(curl, CURLOPT_TIMEOUT, OAUTH_CURL_TIMEOUT);
+  curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
+#endif
   res = curl_easy_perform(curl);
+  curl_slist_free_all(slist);
   if (res) {
     // error
     return NULL;
@@ -323,6 +355,23 @@ char *oauth_curl_post_data_with_callback (const char *u, const char *data, size_
 #define _OAUTH_ENV_HTTPCMD "OAUTH_HTTP_CMD"
 #define _OAUTH_ENV_HTTPGET "OAUTH_HTTP_GET_CMD"
 
+#ifdef OAUTH_CURL_TIMEOUT  
+
+#define cpxstr(s) cpstr(s)
+#define cpstr(s) #s
+
+#ifndef _OAUTH_DEF_HTTPCMD
+# define _OAUTH_DEF_HTTPCMD "curl -sA '"OAUTH_USER_AGENT"' -m "cpxstr(OAUTH_CURL_TIMEOUT)" -d '%p' '%u' "
+//alternative: "wget -q -U 'liboauth-agent/0.1' --post-data='%p' '%u' "
+#endif
+
+#ifndef _OAUTH_DEF_HTTPGET 
+# define _OAUTH_DEF_HTTPGET "curl -sA '"OAUTH_USER_AGENT"' -m "cpxstr(OAUTH_CURL_TIMEOUT)" '%u' "
+//alternative: "wget -q -U 'liboauth-agent/0.1' '%u' "
+#endif
+
+#else // no timeout
+
 #ifndef _OAUTH_DEF_HTTPCMD
 # define _OAUTH_DEF_HTTPCMD "curl -sA '"OAUTH_USER_AGENT"' -d '%p' '%u' "
 //alternative: "wget -q -U 'liboauth-agent/0.1' --post-data='%p' '%u' "
@@ -333,6 +382,8 @@ char *oauth_curl_post_data_with_callback (const char *u, const char *data, size_
 //alternative: "wget -q -U 'liboauth-agent/0.1' '%u' "
 #endif
 
+#endif
+
 #include <stdio.h>
 
 /**
@@ -433,7 +484,7 @@ char *oauth_exec_post (const char *u, const char *p) {
   t1=strstr(cmdtpl, "%p");
   t2=strstr(cmdtpl, "%u");
   if (!t1 || !t2) {
-    fprintf(stderr, "\nliboauth: invalid HTTP command. set the '%s' environement variable.\n\n",_OAUTH_ENV_HTTPCMD);
+    fprintf(stderr, "\nliboauth: invalid HTTP command. set the '%s' environment variable.\n\n",_OAUTH_ENV_HTTPCMD);
     return(NULL);
   }
   // TODO: check if there are exactly two '%' in cmdtpl
@@ -478,7 +529,7 @@ char *oauth_exec_get (const char *u, const char *q) {
   // add URL and post param - error if no '%p' or '%u' present in definition
   t1=strstr(cmdtpl, "%u");
   if (!t1) {
-    fprintf(stderr, "\nliboauth: invalid HTTP command. set the '%s' environement variable.\n\n",_OAUTH_ENV_HTTPGET);
+    fprintf(stderr, "\nliboauth: invalid HTTP command. set the '%s' environment variable.\n\n",_OAUTH_ENV_HTTPGET);
     return(NULL);
   }
   *(++t1)= 's';
@@ -515,7 +566,7 @@ char *oauth_exec_get (const char *u, const char *q) {
  */
 char *oauth_http_get (const char *u, const char *q) {
 #ifdef HAVE_CURL
-  return oauth_curl_get(u,q);
+  return oauth_curl_get(u,q,NULL);
 #elif defined(HAVE_SHELL_CURL)
   return oauth_exec_get(u,q);
 #else 
@@ -523,6 +574,25 @@ char *oauth_http_get (const char *u, const char *q) {
 #endif
 }
 
+/**
+ * do a HTTP GET request, wait for it to finish 
+ * and return the content of the reply.
+ * (requires libcurl)
+ * 
+ * @param u base url to get
+ * @param q query string to send along with the HTTP request or NULL.
+ * @param customheader specify custom HTTP header (or NULL for none)
+ * @return  In case of an error NULL is returned; otherwise a pointer to the
+ * replied content from HTTP server. latter needs to be freed by caller.
+ */
+char *oauth_http_get2 (const char *u, const char *q, const char *customheader) {
+#ifdef HAVE_CURL
+  return oauth_curl_get(u,q,customheader);
+#else 
+  return NULL;
+#endif
+}
+
 /**
  * do a HTTP POST request, wait for it to finish 
  * and return the content of the reply.
@@ -537,7 +607,7 @@ char *oauth_http_get (const char *u, const char *q) {
  */
 char *oauth_http_post (const char *u, const char *p) {
 #ifdef HAVE_CURL
-  return oauth_curl_post(u,p);
+  return oauth_curl_post(u,p,NULL);
 #elif defined(HAVE_SHELL_CURL)
   return oauth_exec_post(u,p);
 #else 
@@ -545,6 +615,28 @@ char *oauth_http_post (const char *u, const char *p) {
 #endif
 }
 
+
+/**
+ * do a HTTP POST request, wait for it to finish 
+ * and return the content of the reply.
+ * (requires libcurl)
+ *
+ * more documentation in oauth.h
+ *
+ * @param u url to query
+ * @param p postargs to send along with the HTTP request.
+ * @param customheader specify custom HTTP header (or NULL for none)
+ * @return  In case of an error NULL is returned; otherwise a pointer to the
+ * replied content from HTTP server. latter needs to be freed by caller.
+ */
+char *oauth_http_post2 (const char *u, const char *p, const char *customheader) {
+#ifdef HAVE_CURL
+  return oauth_curl_post(u,p,customheader);
+#else
+  return NULL;
+#endif
+}
+
 /**
  * http post raw data from file.
  * the returned string needs to be freed by the caller
